package com.annimon.gipgame;

import java.awt.AWTEvent;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import javax.swing.JPanel;

public class GamePanel extends JPanel implements Runnable {
    
    private static final int BONUS_COUNT = 10;
    
    private Background background;
    private PlayerShip player;
    private Bonus[] bonus;
    
    private Thread thread;
    
    private int xStep;
    private int width, height;
    
    public GamePanel() {
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        width = (int) (screenSize.getWidth() * 300 / 683);
        height = (int) screenSize.getHeight();
        setPreferredSize( new Dimension(width, height) );
        setFocusable(true);
        enableEvents(AWTEvent.KEY_EVENT_MASK);
        
        initView();
    }
    
    private void initView() {
        background = new Background();
        player = new PlayerShip();
        player.init(width, height);
        
        Bonus.initBonusSize(width, height);
        bonus = new Bonus[BONUS_COUNT];
        for (int i = 0; i < BONUS_COUNT; i++) {
            bonus[i] = new Bonus();
        }
        
        xStep = 0;
        
        thread = new Thread(this);
        thread.start();
    }
    
    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        g.setColor(Color.BLACK);
        g.fillRect(0, 0, width, height);
        
        background.draw(g);
        player.draw(g);
        if (!player.gameIsFinished()) {
            for (int i = 0; i < BONUS_COUNT; i++) {
                bonus[i].draw(g);
            }
        }
    }
    
    @Override
    public void processKeyEvent(KeyEvent event) {
        final int key = event.getKeyCode();
        if (event.getID() == KeyEvent.KEY_PRESSED) {
            final int MOVE_STEP = 5;
            if (key == KeyEvent.VK_LEFT) {
                xStep = MOVE_STEP;
            } else if (key == KeyEvent.VK_RIGHT) {
                xStep = -MOVE_STEP;
            } else if (key == KeyEvent.VK_ESCAPE) {
                System.exit(0);
            }
            if (event.isShiftDown()) xStep *= 3;
        } else if (event.getID() == KeyEvent.KEY_RELEASED) {
            if ( (key == KeyEvent.VK_LEFT) || (key == KeyEvent.VK_RIGHT) ) {
                xStep = 0;
            }
        }
    }

    private void update() {
        player.updateX(xStep);
        background.update(player.getScore());
        
        if (player.gameIsFinished()) return;
        
        for (int i = 0; i < BONUS_COUNT; i++) {
            bonus[i].update(player);
        }
    }
    
    @Override
    public void run() {
        while(Thread.currentThread() == thread) {
            Fps.startMeasuringDelay();
            update();
            repaint();
            try {
                Thread.sleep(Fps.getDelay());
            } catch (InterruptedException ex) {
                Thread.currentThread().interrupt();
            }
        }
    }
    
}
