package com.annimon.gipgame;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;



public class PlayerShip {
    
    private static final int WIDTH = 5, HEIGHT = 4;
    private static Color[] SHIP_COLORS = {
        Color.WHITE,
        Color.GREEN,
        Color.BLUE,
        Color.CYAN,
        Color.YELLOW,
        Color.DARK_GRAY,
        Color.MAGENTA,
        Color.RED,
        Color.BLACK,
        new Color(0x00, true)
    };
    
    private int SHIP_WIDTH, SHIP_HEIGHT;
    private int TEXT_SIZE;
    
    private int x, y;
    private int scrWidth, scrHeight;
    private int score;
    private BufferedImage shipBitmap;
    
    public PlayerShip() {
        score = 0;
    }
    
    public void init(int w, int h) {
        scrWidth = w;
        scrHeight = h;
        
        SHIP_WIDTH  = w / 16;
        SHIP_HEIGHT = h / 20;
        TEXT_SIZE = 12;
        
        createPlayerShip(SHIP_COLORS[0].getRGB());
        
        x = (w - SHIP_WIDTH) / 2;
        y = h - SHIP_HEIGHT * 2;
    }
    
    public int getScore() {
        return score;
    }
    
    public boolean gameIsFinished() {
        return ( score > (SHIP_COLORS.length * 20) );
    }

    public void draw(Graphics g) {
        // When game is finished we don't need to draw ship.
        if (gameIsFinished()) {
            drawText(g, "Game is finished", true);
            return;
        }
        // Draw ship.
        g.drawImage(shipBitmap, x, y, null);
        // Draw score.
        drawText(g, String.valueOf(score), false);
    }
    
    public void updateX(int value) {
        x -= value;
        
        if (x < 0) x = 0;
        else if (x > ( scrWidth - SHIP_WIDTH )) {
            x = scrWidth - SHIP_WIDTH;
        }
    }
    
    public boolean isCollide(Rectangle2D.Float rectangle) {
        return (rectangle.intersects(x, y, SHIP_WIDTH, SHIP_HEIGHT));
    }
    
    public void changeScore(int value) {
        if (gameIsFinished()) return;
        
        score += value;
        if (score < 0) score = 0;
        else if (score % 20 == 0) {
            // Change color of the ship by score level.
            int shipLevel = Math.min(score / 20, SHIP_COLORS.length - 1);
            createPlayerShip(SHIP_COLORS[shipLevel].getRGB());
        }
    }
    
    private void createPlayerShip(int color) {
        //color |= 0xFF000000;
        BufferedImage bmp = new BufferedImage(WIDTH, HEIGHT, BufferedImage.TYPE_INT_ARGB);
        bmp.setRGB(2, 0, color);
        bmp.setRGB(1, 1, color); bmp.setRGB(3, 1, color);
        bmp.setRGB(1, 2, color); bmp.setRGB(3, 2, color);
        bmp.setRGB(0, 3, color); bmp.setRGB(4, 3, color);
        // Stretch bitmap to calculated size.
        shipBitmap = new BufferedImage(SHIP_WIDTH, SHIP_HEIGHT, BufferedImage.TYPE_INT_ARGB);
        AffineTransform at = new AffineTransform();
        at.scale(SHIP_WIDTH / (float) WIDTH, SHIP_HEIGHT / (float) HEIGHT);
        AffineTransformOp scaleOp = new AffineTransformOp(at, AffineTransformOp.TYPE_NEAREST_NEIGHBOR);
        shipBitmap = scaleOp.filter(bmp, shipBitmap);
    }
    
    private void drawText(Graphics g, String text, boolean vertCenter) {
        g.setColor(Color.YELLOW);
        g.drawString(text, scrWidth / 2 - (text.length()), 10);
    }
}
